@@ -112,6 +112,9 @@ MC7000.factor = [];
 //Set Shift button state to false as default
 MC7000.shift = [false, false, false, false];
 
+// For each side whether the top or bottom deck is active.
+MC7000.topDeckActive = [true, true];
+
 // initialize the PAD Mode to Hot Cue and all others off when starting
 MC7000.PADModeCue = [true, true, true, true];
 MC7000.PADModeCueLoop = [false, false, false, false];
@@ -181,6 +184,10 @@ MC7000.init = function() {
     engine.makeConnection("[Channel3]", "VuMeter", MC7000.VuMeter);
     engine.makeConnection("[Channel4]", "VuMeter", MC7000.VuMeter);
 
+    // Switch to active decks
+    midi.sendShortMsg(MC7000.topDeckActive[0] ? 0x90 : 0x92, 0x08, 0x7F);
+    midi.sendShortMsg(MC7000.topDeckActive[1] ? 0x91 : 0x93, 0x08, 0x7F);
+
     // Platter Ring LED mode
     midi.sendShortMsg(0x90, 0x64, MC7000.modeSingleLED);
     midi.sendShortMsg(0x91, 0x64, MC7000.modeSingleLED);
@@ -494,6 +501,15 @@ MC7000.PadButtons = function(channel, control, value, status, group) {
     var deckOffset = deckNumber - 1;
     var i, j;
 
+    // The following modes are currently unhandled and could be
+    // added as if-branches in the future:
+
+    // - MC7000.PADModeCueLoop
+    // - MC7000.PADModeFlip
+    // - MC7000.PADModeSlicerLoop
+    // - MC7000.PADModeVelSamp
+    // - MC7000.PADModePitch
+
     // activate and clear Hot Cues
     if (MC7000.PADModeCue[deckNumber] && engine.getValue(group, "track_loaded") === 1) {
         for (i = 1; i <= 8; i++) {
@@ -512,10 +528,6 @@ MC7000.PadButtons = function(channel, control, value, status, group) {
                 midi.sendShortMsg(0x94 + deckOffset, 0x1C + i - 1, MC7000.padColor.hotcueoff);
             }
         }
-    } else if (MC7000.PADModeCueLoop[deckNumber]) {
-        return;
-    } else if (MC7000.PADModeFlip[deckNumber]) {
-        return;
     } else if (MC7000.PADModeRoll[deckNumber]) {
         // TODO(all): check for actual beatloop_size and apply back after a PAD Roll
         i = control - 0x14;
@@ -560,8 +572,6 @@ MC7000.PadButtons = function(channel, control, value, status, group) {
         } else {
             midi.sendShortMsg(0x94 + deckOffset, control, MC7000.padColor.sliceron);
         }
-    } else if (MC7000.PADModeSlicerLoop[deckNumber]) {
-        return;
     } else if (MC7000.PADModeSampler[deckNumber]) {
         for (i = 1; i <= 8; i++) {
             if (control === 0x14 + i - 1 && value >= 0x01) {
@@ -588,17 +598,13 @@ MC7000.PadButtons = function(channel, control, value, status, group) {
                 }
             }
         }
-    } else if (MC7000.PADModeVelSamp[deckNumber]) {
-        return;
-    } else if (MC7000.PADModePitch[deckNumber]) {
-        return;
     }
 };
 
 // Shift Button
 MC7000.shiftButton = function(channel, control, value, status, group) {
     var deckOffset = script.deckFromGroup(group) - 1;
-    MC7000.shift[deckOffset] = ! MC7000.shift[deckOffset];
+    MC7000.shift[deckOffset] = value > 0;
     midi.sendShortMsg(0x90 + deckOffset, 0x32,
         MC7000.shift[deckOffset] ? 0x7F : 0x01);
 };
@@ -888,6 +894,26 @@ MC7000.crossFaderCurve = function(control, value) {
     script.crossfaderCurve(value);
 };
 
+// Update state on deck changes
+MC7000.switchDeck = function(channel, control, value, status) {
+    var deckOffset = status - 0x90;
+    var isTopDeck = deckOffset < 2;
+    var side = deckOffset % 2;
+    var previousDeckOffset = (deckOffset + 2) % 4;
+
+    // We need to 'transfer' the shift state when switching decks,
+    // otherwise it will get stuck and result in an 'inverted'
+    // shift after switching back to the deck.
+    // Since the controller switches immediately upon pressing down,
+    // we only do this when value is high.
+
+    if (value === 0x7F && MC7000.topDeckActive[side] !== isTopDeck) {
+        MC7000.topDeckActive[side] = isTopDeck;
+        MC7000.shift[deckOffset] = MC7000.shift[previousDeckOffset];
+        MC7000.shift[previousDeckOffset] = false;
+    }
+};
+
 // Set FX wet/dry value
 MC7000.fxWetDry = function(channel, control, value, status, group) {
     var numTicks = (value < 0x64) ? value : (value - 128);
