@@ -146,6 +146,9 @@ TraktorS3.Controller = function() {
     // If true, channel 4 is in input mode
     this.channel4InputMode = false;
 
+    // Represents the first-pressed deck switch button, used for tracking deck clones.
+    this.deckSwitchPressed = "";
+
     // callbacks
     this.samplerCallbacks = [];
 };
@@ -1791,6 +1794,20 @@ TraktorS3.Controller.prototype.headphoneHandler = function(field) {
 
 TraktorS3.Controller.prototype.deckSwitchHandler = function(field) {
     if (field.value === 0) {
+        if (this.deckSwitchPressed === field.group) {
+            this.deckSwitchPressed = "";
+        }
+        return;
+    }
+
+    if (this.deckSwitchPressed === "") {
+        this.deckSwitchPressed = field.group;
+    } else {
+        // If a different deck switch is already pressed, do an instant double and do not select the
+        // deck.
+        var cloneFrom = this.Channels[this.deckSwitchPressed];
+        var cloneFromNum = cloneFrom.parentDeck.deckNumber;
+        engine.setValue(field.group, "CloneFromDeck", cloneFromNum);
         return;
     }
 
@@ -1909,8 +1926,8 @@ TraktorS3.Controller.prototype.registerOutputPackets = function() {
     engine.connectControl("[Master]", "maximize_library", TraktorS3.bind(TraktorS3.Controller.prototype.maximizeLibraryOutput, this));
 
     // Master VuMeters
-    this.masterVuMeter.VuMeterL.connection = engine.makeUnbufferedConnection("[Master]", "VuMeterL", TraktorS3.bind(TraktorS3.Controller.prototype.masterVuMeterHandler, this));
-    this.masterVuMeter.VuMeterR.connection = engine.makeUnbufferedConnection("[Master]", "VuMeterR", TraktorS3.bind(TraktorS3.Controller.prototype.masterVuMeterHandler, this));
+    this.masterVuMeter.VuMeterL.connection = engine.makeConnection("[Master]", "VuMeterL", TraktorS3.bind(TraktorS3.Controller.prototype.masterVuMeterHandler, this));
+    this.masterVuMeter.VuMeterR.connection = engine.makeConnection("[Master]", "VuMeterR", TraktorS3.bind(TraktorS3.Controller.prototype.masterVuMeterHandler, this));
     this.linkChannelOutput("[Master]", "PeakIndicatorL", TraktorS3.bind(TraktorS3.Controller.prototype.peakOutput, this));
     this.linkChannelOutput("[Master]", "PeakIndicatorR", TraktorS3.bind(TraktorS3.Controller.prototype.peakOutput, this));
     this.guiTickConnection = engine.makeConnection("[Master]", "guiTick50ms", TraktorS3.bind(TraktorS3.Controller.prototype.guiTickHandler, this));
