@@ -544,8 +544,7 @@ void MixxxMainWindow::initialize(QApplication* pApp, const CmdlineArgs& args) {
     QFile file(":/skins/default.qss");
     if (file.open(QIODevice::ReadOnly)) {
         QByteArray fileBytes = file.readAll();
-        QString style = QString::fromLocal8Bit(fileBytes.constData(),
-                                               fileBytes.length());
+        QString style = QString::fromLocal8Bit(fileBytes);
         setStyleSheet(style);
     } else {
         qWarning() << "Failed to load default skin styles!";
